// src/index.ts
import { EntryPlugin } from "webpack";
import {
  ensureAbsolute as ensureAbsolute2,
  isString,
  debug as debug3,
  DEVTOOLS_VIRTUAL_ID,
  TAILWIND_ENTRY_VIRTUAL_ID
} from "tailwindcss-webpack-plugin-utils";

// src/constants.ts
import { dirname, join } from "path";
import { fileURLToPath } from "url";
var PLUGIN_NAME = "tailwindcss-webpack-plugin";
var DEFAULT_DEVTOOLS_PORT = 9999;
var DEFAULT_DEVTOOLS_HOST = "127.0.0.1";
var PACKAGE_DIR = typeof __dirname !== "undefined" ? __dirname : dirname(fileURLToPath(import.meta.url));
var COMPILE_ENTRY_PATH = join(
  PACKAGE_DIR,
  "./templates/_compile-entry_.js"
);
var TAILWIND_ENTRY_PATH = join(
  PACKAGE_DIR,
  "./templates/_tailwind_.css"
);
var DEVTOOLS_ENTRY_PATH = join(
  PACKAGE_DIR,
  "./templates/_tailwind-devtools_.js"
);
var DEVTOOLS_LOADER_PATH = join(
  PACKAGE_DIR,
  "./loaders/devtools-loader.js"
);
var CSS_LOADER_PATH = join(PACKAGE_DIR, "./loaders/css-loader.js");

// src/util.ts
import { utimesSync } from "fs";
import {
  BaseTailwindService,
  debug as debug2,
  ensureAbsolute
} from "tailwindcss-webpack-plugin-utils";

// src/devtools/server.ts
import http from "http";
import { getPort } from "get-port-please";
import {
  promiseSingleton,
  debug,
  createDevtoolsMiddleware
} from "tailwindcss-webpack-plugin-utils";
var DevtoolsServer = class {
  constructor(options, compiler) {
    this._ensureStart = promiseSingleton(
      () => new Promise(async (resolve) => {
        this.port = await getPort(this.port);
        if (!this._listen) {
          this.server.listen(this.port, this.host, () => {
            this._listen = true;
            debug(`[devtools-server]: listen at ${this.host}:${this.port}`);
            process.on("exit", () => {
              this.close();
            });
            resolve({
              port: this.port,
              host: this.host
            });
          });
        }
      })
    );
    var _a, _b;
    this.host = ((_a = options.devtools) == null ? void 0 : _a.host) ?? DEFAULT_DEVTOOLS_HOST;
    this.port = ((_b = options.devtools) == null ? void 0 : _b.port) ?? DEFAULT_DEVTOOLS_PORT;
    this.server = http.createServer(async (req, res) => {
      const devtoolsMiddleware = createDevtoolsMiddleware(
        compiler.$tailwind.service
      );
      devtoolsMiddleware(req, res);
    });
    this.server.on("error", (err) => {
      debug(`[devtools-server]: listen err`, err);
      throw err;
    });
    this.service = compiler.$tailwind;
    this._listen = false;
  }
  async ensureStart() {
    debug(`[devtools-server]: ensure start server`);
    return this._ensureStart();
  }
  close(callback) {
    if (this._listen) {
      debug(`[devtools-server]: close`);
      this.server.close((err) => callback == null ? void 0 : callback(err));
    } else {
      callback == null ? void 0 : callback();
    }
  }
};

// src/util.ts
var Service = class extends BaseTailwindService {
  constructor(options) {
    super(options);
  }
  invalidateCssModule() {
    var _a, _b;
    debug2(
      `invalidate virtual module ${((_a = this.options) == null ? void 0 : _a.entry) ?? TAILWIND_ENTRY_PATH}`
    );
    const fullPath = ensureAbsolute(((_b = this.options) == null ? void 0 : _b.entry) ?? TAILWIND_ENTRY_PATH);
    const timestamp = Date.now();
    utimesSync(fullPath, timestamp, timestamp);
  }
};
var ensureService = async (compiler, options) => {
  if (compiler.$tailwind) {
    return;
  }
  debug2(`ensure compiler.$tailwind options -> `, options);
  compiler.$tailwind = {
    dirty: /* @__PURE__ */ new Set(),
    server: new DevtoolsServer(options, compiler),
    service: new Service(options)
  };
};

// src/index.ts
var TailwindCSSWebpackPlugin = class {
  constructor(options) {
    this.options = options ?? {};
  }
  apply(compiler) {
    var _a, _b;
    compiler.options.resolve.alias = {
      [DEVTOOLS_VIRTUAL_ID]: DEVTOOLS_ENTRY_PATH,
      [TAILWIND_ENTRY_VIRTUAL_ID]: ensureAbsolute2(
        ((_a = this.options) == null ? void 0 : _a.entry) ?? TAILWIND_ENTRY_PATH
      ),
      ...compiler.options.resolve.alias
    };
    compiler.options.module.rules.unshift({
      enforce: "pre",
      include: (resource) => resource.includes(TAILWIND_ENTRY_VIRTUAL_ID) || /\.css$/.test(resource) && !/node_modules/.test(resource),
      loader: CSS_LOADER_PATH
    });
    compiler.options.module.rules.unshift({
      enforce: "pre",
      include: (resource) => resource.includes(DEVTOOLS_VIRTUAL_ID),
      loader: DEVTOOLS_LOADER_PATH
    });
    if (!isString((_b = this.options) == null ? void 0 : _b.entry)) {
      new EntryPlugin(compiler.context, COMPILE_ENTRY_PATH, {
        name: void 0
      }).apply(compiler);
    }
    compiler.hooks.invalid.tap(PLUGIN_NAME, (file) => {
      var _a2;
      debug3(`[compiler ${compiler.name ?? ""}] invalid hook file ->`, file);
      if (!file) {
        file = "config-file";
      }
      if (file === ensureAbsolute2(((_a2 = this.options) == null ? void 0 : _a2.entry) ?? TAILWIND_ENTRY_PATH)) {
        return;
      }
      if (!compiler.$tailwind.dirty.has(file)) {
        compiler.$tailwind.dirty.add(file);
        compiler.$tailwind.service.invalidateCssModule();
      }
    });
    compiler.hooks.afterCompile.tap(PLUGIN_NAME, () => {
      compiler.$tailwind.service.clearChangedContent();
    });
    compiler.hooks.beforeCompile.tapPromise(PLUGIN_NAME, async () => {
      debug3(`[compiler ${compiler.name ?? ""}] beforeCompiler hook`);
      await ensureService(compiler, this.options);
    });
    compiler.hooks.watchRun.tapPromise(PLUGIN_NAME, async () => {
      debug3(`[compiler ${compiler.name ?? ""}] watchRun hook`);
      await ensureService(compiler, this.options);
    });
    compiler.hooks.beforeRun.tapPromise(PLUGIN_NAME, async () => {
      debug3(`[compiler ${compiler.name ?? ""}] beforeRun hook`);
      await ensureService(compiler, this.options);
    });
    compiler.hooks.compilation.tap(PLUGIN_NAME, async (compilation) => {
      debug3(`[compiler ${compiler.name ?? ""}] compilation hook`);
      await ensureService(compilation.compiler, this.options);
    });
    compiler.hooks.shutdown.tapAsync(PLUGIN_NAME, (callback) => {
      debug3(`[compiler ${compiler.name ?? ""}] shutdown hook`);
      compiler.$tailwind.server.close(callback);
    });
  }
};
export {
  TailwindCSSWebpackPlugin
};
